package nand.apps.chat

import android.Manifest.permission.POST_NOTIFICATIONS
import android.app.Notification
import android.app.PendingIntent
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.content.pm.ServiceInfo
import android.os.Build
import androidx.core.app.*
import androidx.lifecycle.LifecycleService
import androidx.lifecycle.lifecycleScope
import chat.seers.app.R
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.engine.ChatEngine
import nand.apps.chat.event.ChatEventHandler
import nand.apps.chat.log.getLogger
import nand.apps.chat.model.event.LogoutEvent
import nand.apps.chat.notification.*
import nand.apps.chat.util.getStringBlocking
import org.koin.android.ext.android.getKoin
import seers.composeapp.generated.resources.*

private const val CHANNEL_ID = "$APP_NAME Service"
private const val NOTIFICATION_ID = 69420
private const val INTENT_ACTION_LOGOUT = "$APP_NAME Logout"
private const val NOTIFICATION_PERMISSION_WAIT_INTERVAL = 30_000L // Check every 30 seconds

class ChatEngineService : LifecycleService() {

    private val chatEngine: ChatEngine by getKoin().inject()
    private val eventHandler: ChatEventHandler by getKoin().inject()
    private val notificationManager by lazy { NotificationManagerCompat.from(this) }

    private val intentReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            intent?.let { onIntent(it) }
        }
    }

    private val logger = getLogger("ChatEngineService")

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        super.onStartCommand(intent, flags, startId)
        if (!isNotificationVisible()) setupNotification()
        registerIntentReceiver()
        chatEngine.start(lifecycleScope)
        logger.debug("Started chat engine service")
        return START_NOT_STICKY
    }

    override fun onDestroy() {
        super.onDestroy()
        chatEngine.stop()
        unregisterReceiver(intentReceiver)
        logger.debug("Stopped chat engine service")
    }

    private fun setupNotification() {
        val channel = NotificationChannelCompat.Builder(
            CHANNEL_ID,
            NotificationManagerCompat.IMPORTANCE_LOW,
        ).run {
            setName(APP_NAME)
            build()
        }
        notificationManager.createNotificationChannel(channel)
        val foregroundIntent = PendingIntentCompat.getActivity(this, 0, Intent(this, MainActivity::class.java), 0, true)
        val notification = NotificationCompat.Builder(this, CHANNEL_ID)
            .setSmallIcon(R.drawable.ic_launcher_monochrome)
            .setContentIntent(foregroundIntent)
            .setContentTitle(getStringBlocking(Res.string.login_notification_message, APP_NAME))
            .addAction(getLogoutNotificationAction())
            .build()

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            startForeground(
                NOTIFICATION_ID,
                notification,
                ServiceInfo.FOREGROUND_SERVICE_TYPE_SPECIAL_USE,
            )
        } else {
            startForeground(NOTIFICATION_ID, notification)
        }

        if (!isNotificationsAllowed()) {
            logger.debug("Notification permission has not been granted yet - waiting for allow...")
            lifecycleScope.launch { waitForNotificationPermission(notification) }
        }
    }

    private fun isNotificationsAllowed() =
        ActivityCompat.checkSelfPermission(this, POST_NOTIFICATIONS) == PackageManager.PERMISSION_GRANTED

    private fun isNotificationVisible() =
        notificationManager.activeNotifications.find { it.id == NOTIFICATION_ID } != null

    private suspend fun waitForNotificationPermission(notification: Notification) {
        while (true) {
            delay(NOTIFICATION_PERMISSION_WAIT_INTERVAL)
            if (isNotificationsAllowed()) {
                if (!isNotificationVisible()) {
                    notificationManager.notify(NOTIFICATION_ID, notification)
                    logger.debug("Showing service notification")
                }
                break
            }
        }
    }

    private fun getLogoutNotificationAction(): NotificationCompat.Action {
        val logoutPendingIntent = PendingIntent.getBroadcast(
            this,
            AndroidNotificationRequest.LOGOUT.code,
            Intent(INTENT_ACTION_LOGOUT).setPackage(packageName),
            PendingIntent.FLAG_UPDATE_CURRENT or PendingIntent.FLAG_MUTABLE,
        )
        return NotificationCompat.Action.Builder(
            R.drawable.ic_launcher_monochrome,
            getStringBlocking(Res.string.logout_confirmation_dialog_title),
            logoutPendingIntent
        ).build()
    }

    private fun registerIntentReceiver() {
        val receiverFlags = when {
            Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU -> Context.RECEIVER_NOT_EXPORTED
            else -> 0
        }
        registerReceiver(intentReceiver, IntentFilter(INTENT_ACTION_LOGOUT), receiverFlags)
    }

    private fun onIntent(intent: Intent) {
        if (intent.action == INTENT_ACTION_LOGOUT) {
            eventHandler += LogoutEvent
        }
    }
}