package nand.apps.chat.engine.tox.friend

import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import nand.apps.chat.engine.tox.ToxFileEncryption
import nand.apps.chat.io.ChatDirectory
import nand.apps.chat.io.ChatFileSystem
import nand.apps.chat.log.getLogger
import nand.apps.chat.model.uid.FriendRequestUid
import nand.apps.chat.model.uid.UserUid
import nand.apps.chat.model.user.*
import nand.apps.chat.repo.ChatAvatarRepo
import okio.Path

private const val REQUEST_PREFIX = "request-"

class ToxFriendSaver(
    private val fileSystem: ChatFileSystem,
    private val avatarRepo: ChatAvatarRepo,
) {

    private lateinit var directory: ChatDirectory
    private var encryption: ToxFileEncryption? = null
    private val json = Json { ignoreUnknownKeys = true }
    private val logger = getLogger("ToxFriendSaver")

    fun init(
        dirName: String,
        encryption: ToxFileEncryption?,
    ) {
        directory = fileSystem.getProfileDirectory("$dirName/friends")
        this.encryption = encryption
    }

    fun save(friend: UserData): Boolean {
        val data = try {
            json.encodeToString(friend.toPersistedUserData()).encodeToByteArray()
        } catch (e: Exception) {
            logger.error("Failed to save friend data", e)
            return false
        }
        return directory.writeToFile(
            fileName = friend.uid.value,
            data = data,
            encryption = encryption,
        )
    }

    fun save(friendRequest: FriendRequest): Boolean {
        val data = try {
            json.encodeToString(friendRequest.toPersistedFriendRequest()).encodeToByteArray()
        } catch (e: Exception) {
            logger.error("Failed to save friend request", e)
            return false
        }
        return directory.writeToFile(
            fileName = "$REQUEST_PREFIX${friendRequest.uid.value}",
            data = data,
            encryption = encryption,
        )
    }

    fun remove(friendUid: UserUid) {
        directory.removeFile(friendUid.value)
    }

    fun remove(friendRequestUid: FriendRequestUid) {
        directory.removeFile("$REQUEST_PREFIX${friendRequestUid.value}")
    }

    fun loadFriends() = directory.listFiles().filter { !it.isFriendRequest() }.mapNotNull { loadFriend(it) }

    fun loadFriendRequests() = directory.listFiles().filter { it.isFriendRequest() }.mapNotNull { loadFriendRequest(it) }

    private fun Path.isFriendRequest() = name.startsWith(REQUEST_PREFIX)

    private fun loadFriend(friendPath: Path): UserData? {
        val data = directory.readFromFile(
            fileName = friendPath.name,
            encryption = encryption,
        ) ?: return null
        val friend = try {
            json.decodeFromString<PersistedUserData>(data.decodeToString())
        } catch (e: Exception) {
            logger.error("Failed to read friend data: $friendPath", e)
            return null
        }
        return friend.toUserData(avatarRepo)
    }

    private fun loadFriendRequest(friendPath: Path): FriendRequest? {
        // Safeguard against loading both friend and friend request
        val friendFileName = friendPath.name.replace(REQUEST_PREFIX, "")
        if (friendFileName in directory) {
            directory.removeFile(friendPath.name)
            return null
        }
        val data = directory.readFromFile(
            fileName = friendPath.name,
            encryption = encryption,
        ) ?: return null
        val request = try {
            json.decodeFromString<PersistedFriendRequest>(data.decodeToString())
        } catch (e: Exception) {
            logger.error("Failed to read friend request: $friendPath", e)
            return null
        }
        return request.toFriendRequest()
    }
}