package nand.apps.chat.ui.chat.screen

import nand.apps.chat.model.call.CallData
import nand.apps.chat.model.file.ChatFileTransfer
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.group.ChatGroupVisibility
import nand.apps.chat.model.message.ChatMessage
import nand.apps.chat.model.message.UnreadMessageCounts
import nand.apps.chat.model.uid.*
import nand.apps.chat.model.user.UserData
import nand.apps.chat.ui.chat.message.MessageInputState
import nand.apps.chat.util.getStringBlocking
import nand.apps.chat.util.takeIfNotBlank
import seers.composeapp.generated.resources.*

data class ChatScreenState(
    val conversationUid: ContactUid? = null,
    val messages: List<ChatMessage> = emptyList(),
    val self: UserData = UserData.BLANK,
    val friends: Map<UserUid, UserData> = emptyMap(),
    val groups: Map<GroupUid, ChatGroupData> = emptyMap(),
    val unreadCounts: Map<ContactUid, UnreadMessageCounts> = emptyMap(),
    val fileTransfers: Map<FileTransferUid, ChatFileTransfer> = emptyMap(),
    val calls: Map<ContactUid, CallData> = emptyMap(),
    val callActivity: Map<ContactUid, Boolean> = emptyMap(),
    val inputState: MessageInputState = MessageInputState(),
) {
    
    val conversationUser = when (conversationUid) {
        is UserUid -> friends[conversationUid]
        is GroupPeerUid -> groups[conversationUid.groupUid]?.peers?.get(conversationUid.peerUid)
        else -> null
    }
    val conversationGroup = (conversationUid as? GroupUid)?.let { groups[it] }

    val conversationTitle get() = when (conversationUid) {
        is UserUid,
        is GroupPeerUid -> conversationUser?.displayName
        is GroupUid -> conversationGroup?.title?.takeIfNotBlank() ?: getStringBlocking(Res.string.group_untitled)
        is FriendRequestUid -> getStringBlocking(Res.string.friend_request)
        else -> null
    }
    
    val isConversationOnline = when (conversationUid) {
        is UserUid,
        is GroupPeerUid -> conversationUser?.isOnline == true
        is GroupUid -> conversationGroup?.isOnline == true
        else -> true
    }

    val selfUid get() = conversationGroup?.selfUid ?: self.uid

    val conversationCall = calls[conversationUid]

    fun getDisplayIndex(index: Int) = messages.size - index - 1

    fun getPeerUnreadCount(groupUid: GroupUid): Int {
        return if (groups[groupUid]?.visibility == ChatGroupVisibility.PUBLIC) {
            unreadCounts.entries.sumOf { (uid, value) ->
                if (uid is GroupPeerUid && groupUid == uid.groupUid) {
                    value.directMessages
                } else {
                    0
                }
            }
        } else {
            0
        }
    }

    fun findUser(message: ChatMessage) = when {
        message.isGroupMessage -> null
        message.senderUid == self.uid -> self
        message.conversationUid is GroupUid -> message.findGroupPeer(message.conversationUid)
        message.conversationUid is GroupPeerUid -> message.findGroupPeer(message.conversationUid.groupUid)
        else -> friends[message.senderUid]
    }
    
    fun findGroup(message: ChatMessage) = when {
        message.conversationUid is GroupUid -> groups[message.conversationUid]
        else -> null
    }

    private fun ChatMessage.findGroupPeer(conversationUid: GroupUid) = groups[conversationUid]?.let { group ->
        group.peers[senderUid]?.let { peer ->
            if (senderUid == group.selfUid) {
                self
            } else {
                friends[senderUid]
            }?.copy(
                name = peer.name,
                role = peer.role,
                protocol = peer.protocol,
            ) ?: peer
        }
    }
}