package nand.apps.chat.ui.contacts

import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.SnackbarHostState
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.PhotoCamera
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardCapitalization
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.TextFieldValue
import nand.apps.chat.camera.CameraLauncher
import nand.apps.chat.model.event.TakePhotoQrCode
import nand.apps.chat.ui.provider.LocalChatEngineConstraints
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.snackbar.show
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.SimpleTextFieldButton
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.remove
import nand.apps.chat.util.takeIfUnderLength
import nand.apps.chat.util.uppercase
import org.jetbrains.compose.resources.stringResource
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*
import seers.composeapp.generated.resources.Res
import seers.composeapp.generated.resources.add_contact_dialog_address_label

@Composable
fun ContactAddressInputField(
    address: TextFieldValue,
    isPublicAddress: Boolean,
    onValueChange: (TextFieldValue) -> Unit,
    successSnackbar: SnackbarHostState,
    errorSnackbar: SnackbarHostState,
    isError: Boolean = false,
) {
    val cameraLauncher: CameraLauncher by getKoin().inject()
    val eventHandler = LocalEventHandler.current
    val constraints = LocalChatEngineConstraints.current

    val addressRegex = remember(constraints) {
        Regex("[${constraints.uidCharacterRange}]+")
    }
    val addressRegexInv = remember(constraints) {
        Regex("[^${constraints.uidCharacterRange}]+")
    }

    // Listen for QR code updates from camera
    if (cameraLauncher.isSupported) {
        LaunchedEffect(Unit) {
            eventHandler.onEvent.collect { event ->
                if (event is TakePhotoQrCode) {
                    val code = event.code.substringAfter(':') // qTox puts "tox:" before the address in QR code
                    val expectedLength = when {
                        isPublicAddress -> constraints.uidLength
                        else -> constraints.addressLength
                    }
                    if (code.matches(addressRegex) && code.length == expectedLength) {
                        onValueChange(TextFieldValue(code))
                        successSnackbar.show(Res.string.add_contact_dialog_qr_photo_success)
                    } else {
                        errorSnackbar.show(Res.string.add_contact_dialog_qr_photo_error)
                    }
                }
            }
        }
    }

    Row(
        modifier = Modifier.fillMaxWidth(),
    ) {
        SimpleTextField(
            value = address,
            onValueChange = {
                onValueChange(it
                    .takeIfUnderLength(constraints.addressLength, address)
                    .remove(addressRegexInv)
                    .uppercase())
            },
            labelText = stringResource(Res.string.add_contact_dialog_address_label),
            isError = isError,
            keyboardOptions = KeyboardOptions(
                autoCorrectEnabled = false,
                capitalization = KeyboardCapitalization.Characters,
                keyboardType = KeyboardType.Ascii,
                imeAction = ImeAction.Next
            ),
            modifier = Modifier.weight(1f),
        )
        if (cameraLauncher.isSupported) {
            SimpleTextFieldButton(
                buttonIcon = Icons.Outlined.PhotoCamera,
                buttonDescription = stringResource(Res.string.add_contact_dialog_qr_photo),
                onClick = { cameraLauncher.launchForQrCode() },
                modifier = Modifier.padding(
                    start = ChatTheme.dimens.spacingMedium,
                    top = ChatTheme.dimens.spacingMedium
                ),
            )
        }
    }
}