package nand.apps.chat.ui.dialog

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.text.selection.SelectionContainer
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.FilterQuality
import androidx.compose.ui.platform.LocalClipboardManager
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.compose.ui.window.Dialog
import coil3.compose.AsyncImage
import kotlinx.coroutines.delay
import nand.apps.chat.image.qrcode.QrCodeImageData
import nand.apps.chat.repo.SelfProfileRepo
import nand.apps.chat.ui.button.IconButton
import nand.apps.chat.ui.provider.LocalContentShare
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*

@Composable
fun ContactAddressDialog(
    address: String,
    description: String,
    onClose: () -> Unit,
) {
    val clipboardManager = LocalClipboardManager.current
    val contentShare = LocalContentShare.current
    val selfProfileRepo: SelfProfileRepo by getKoin().inject()
    var currentAddress by remember { mutableStateOf(address) }
    var isCopied by remember { mutableStateOf(false) }
    LaunchedEffect(isCopied) {
        if (isCopied) {
            delay(5000L)
            isCopied = false
        }
    }
    Dialog(
        onDismissRequest = onClose,
    ) {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
            modifier = Modifier
                .background(
                    color = ChatTheme.colors.background,
                    shape = ChatTheme.shapes.small,
                )
                .padding(
                    top = ChatTheme.dimens.spacingXLarge,
                    bottom = ChatTheme.dimens.spacingMedium,
                    start = ChatTheme.dimens.spacingLarge,
                    end = ChatTheme.dimens.spacingLarge,
                )
        ) {
            TextLine(
                text = description,
                style = ChatTheme.typography.h6
            )
            AsyncImage(
                model = QrCodeImageData(
                    value = currentAddress,
                    color = ChatTheme.colors.onSurface,
                ),
                contentDescription = description,
                filterQuality = FilterQuality.High,
                modifier = Modifier.size(ChatTheme.dimens.imageSmall),
            )
            SelectionContainer {
                Text(
                    text = currentAddress,
                    color = ChatTheme.colors.onSurface,
                    style = ChatTheme.typography.caption.copy(
                        letterSpacing = (1.5).sp,
                        fontFamily = FontFamily.Monospace,
                    ),
                    modifier = Modifier.widthIn(
                        min = 0.dp,
                        max = ChatTheme.dimens.imageSmall,
                    )
                )
            }
            Row(
                horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingLarge),
            ) {
                IconButton(
                    icon = Icons.Outlined.Close,
                    contentDescription = stringResource(Res.string.close),
                    onClick = onClose,
                )
                if (currentAddress == selfProfileRepo.getSelfAddress()) {
                    IconButton(
                        icon = Icons.Outlined.Refresh,
                        contentDescription = stringResource(Res.string.reset),
                        onClick = {
                            selfProfileRepo.resetSelfAddress()
                            currentAddress = selfProfileRepo.getSelfAddress()
                        },
                    )
                }
                IconButton(
                    icon = when {
                        isCopied -> Icons.Outlined.Check
                        else -> Icons.Outlined.ContentCopy
                    },
                    contentDescription = stringResource(Res.string.copy_address),
                    onClick = {
                        clipboardManager.setText(AnnotatedString(currentAddress))
                        isCopied = true
                    },
                )
                if (contentShare.isSupported(currentAddress)) {
                    IconButton(
                        icon = Icons.Outlined.Share,
                        contentDescription = stringResource(Res.string.share),
                        onClick = { contentShare.share(currentAddress) },
                    )
                }
            }
        }
    }
}