package nand.apps.chat.ui.file

import com.hoc081098.kmp.viewmodel.ViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.IO
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nand.apps.chat.io.*
import nand.apps.chat.model.event.*
import nand.apps.chat.platform.getPlatform
import nand.apps.chat.event.ChatEventHandler
import nand.apps.chat.ui.util.collect
import nand.apps.chat.util.fifoSharedFlow
import nand.apps.chat.util.getStringBlocking
import okio.FileSystem
import okio.Path
import okio.Path.Companion.toPath
import seers.composeapp.generated.resources.*

class ChatFilePickerViewModel(
    private val fileSystem: FileSystem,
    private val filePaths: ChatFilePaths,
    private val eventHandler: ChatEventHandler,
    private val options: ChatFilePickerOptions,
) : ViewModel() {

    val actionData = fifoSharedFlow<ChatFilePickerAction>()
    val stateData = MutableStateFlow(
        ChatFilePickerState(
            directory = getDefaultDirectory(),
            showHiddenFiles = options.showHiddenFiles,
        )
    )
    private var state get() = stateData.value
        set(value) { stateData.tryEmit(value) }
    
    private val backStack = mutableListOf<String>()
    
    init {
        // Open and reload event handler (primarily used for mobile)
        eventHandler += OpenFileBrowserEvent
        collect(eventHandler.onEvent) { event ->
            if (event is ReloadFileBrowserEvent) {
                refresh()
            }
        }
    }
    
    fun setDirectory(path: String, addToBackStack: Boolean = true) {
        if (addToBackStack && state.directory != path) {
            backStack += state.directory
        }
        state = state.copy(directory = path)
        refresh()
    }
    
    fun selectFile(file: ChatFileMetadata) {
        val useSingleClick = getPlatform().isMobile && file.isDirectory && options.mode == ChatFilePickerMode.FILES_ONLY
        if (state.selectedFile == file || useSingleClick) {
            if (file.isDirectory) {
                setDirectory(file.path)
            } else {
                confirmFile(file)
            }
        } else {
            state = state.copy(selectedFile = file)
        }
    }
    
    fun confirmFile(
        file: ChatFileMetadata? = state.selectedFile,
        isOverwriteAllowed: Boolean = false,
    ) {
        file?.let {
            if (options.isOpenDialog || isOverwriteAllowed) {
                actionData.tryEmit(ConfirmFileAction(it.path))
            } else {
                state = state.copy(overwriteFile = it)
            }
        }
    }

    fun confirmFile(fileName: String, isOverwriteAllowed: Boolean = false) {
        val name = fileName.trimPath()
        val path = when {
            name.isFilePath() -> name
            else -> state.directory + Path.DIRECTORY_SEPARATOR + name
        }.toPath()
        when {
            name.isBlank() -> return
            fileSystem.tryExists(path) -> {
                val metadata = fileSystem.getFileMetadata(path) ?: return
                when {
                    metadata.isSelectable() -> confirmFile(metadata, isOverwriteAllowed)
                    metadata.isDirectory -> {
                        setDirectory(path.toString())
                        actionData.tryEmit(SetFileNameAction(""))
                        search("")
                    }
                }
            }
            !options.isOpenDialog -> {
                actionData.tryEmit(ConfirmFileAction(path.toString()))
            }
            name.isFilePath() -> actionData.tryEmit(
                ShowErrorSnackbarAction(getStringBlocking(Res.string.file_picker_path_doesnt_exist_message))
            )
        }
    }
    
    fun toggleHiddenFiles() {
        state = state.copy(showHiddenFiles = !state.showHiddenFiles)
        refresh()
    }
    
    fun search(text: String) {
        val filter = text.trimPath().run { if (isFilePath()) "" else text }
        if (state.searchFilter != filter) {
            state = state.copy(searchFilter = filter)
            refresh()
        }
    }
    
    fun setSortType(sortType: ChatFileSortType) {
        var currentSortType = state.sortType
        var sortReversed = state.sortReversed
        if (sortType != currentSortType) {
            currentSortType = sortType
            sortReversed = false
        } else {
            sortReversed = !sortReversed
        }
        state = state.copy(
            sortType = currentSortType,
            sortReversed = sortReversed,
        )
    }
    
    fun onBackButton() {
        when {
            backStack.isNotEmpty() -> setDirectory(
                path = backStack.removeLast(),
                addToBackStack = false,
            )
            else -> actionData.tryEmit(CancelFilePickerAction)
        }
    }
    
    fun refresh(directory: String = state.directory) {
        viewModelScope.launch {
            refreshFileList(directory)
        }
    }
    
    fun dismissOverwriteDialog() {
        state = state.copy(overwriteFile = null)
    }

    private fun String.trimPath() = trim { it == '"' || it.isWhitespace() }

    private fun ChatFileMetadata.isSelectable() =
        isDirectory && options.mode != ChatFilePickerMode.FILES_ONLY ||
                !isDirectory && options.mode != ChatFilePickerMode.DIRECTORIES_ONLY

    private suspend fun refreshFileList(directory: String) {
        val state = state
        val showHiddenFiles = state.showHiddenFiles
        val sortType = state.sortType
        val sortReversed = state.sortReversed
        val searchText = state.searchFilter
        stateData.emit(state.copy(isLoading = true))
        val sorted = withContext(Dispatchers.IO) {
            fileSystem.listOrNull(directory.toPath())?.mapNotNull {
                fileSystem.getFileMetadata(it)
            }?.filter { file ->
                val isHidden = !showHiddenFiles && file.isHidden
                val matchesExtension = file.isDirectory || options.extensions == null || file.extension in options.extensions
                when {
                    isHidden -> false
                    options.mode == ChatFilePickerMode.DIRECTORIES_ONLY && !file.isDirectory -> false
                    !matchesExtension -> false
                    searchText.isBlank() -> true
                    searchText.length == 1 -> file.name.startsWith(searchText, ignoreCase = true)
                    else -> file.name.contains(searchText, ignoreCase = true)
                }
            }?.sortedWith(
                getFileComparator(
                    type = sortType,
                    isReversed = sortReversed,
                )
            ).orEmpty()
        }
        var selectedFile = state.selectedFile
        if (selectedFile?.path?.substringBeforeLast(Path.DIRECTORY_SEPARATOR) != directory) {
            selectedFile = null
        }
        stateData.emit(
            state.copy(
                files = sorted,
                selectedFile = selectedFile,
                isLoading = false,
            )
        )
    }
    
    private fun getDefaultDirectory(): String {
        return if (options.directory != null && fileSystem.isValidDirectory(options.directory)) {
            options.directory
        } else {
            filePaths.homePath.toString()
        }
    }
}