package nand.apps.chat.ui.friends

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.SnackbarHostState
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.ImeAction
import nand.apps.chat.ui.contacts.ContactAddressInputField
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.friends.AddFriendDialogAction.*
import nand.apps.chat.ui.provider.LocalChatEngineConstraints
import nand.apps.chat.ui.snackbar.SimpleSnackbarHost
import nand.apps.chat.ui.snackbar.show
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.viewmodel.composableViewModel
import nand.apps.chat.util.takeIfNotBlank
import nand.apps.chat.util.takeIfUnderLength
import org.jetbrains.compose.resources.stringArrayResource
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

@Composable
fun AddFriendDialog(
    onDismiss: () -> Unit,
    viewModel: AddFriendDialogViewModel = composableViewModel(),
) {
    var name by rememberTextFieldValue()
    var address by rememberTextFieldValue()
    var message by rememberTextFieldValue(
        initialValue = stringArrayResource(Res.array.add_friend_dialog_message_defaults)
            .random()
            .replace("%s", viewModel.selfName),
    )
    var hasAddressError by remember { mutableStateOf(false) }
    val errorSnackbars = remember { SnackbarHostState() }
    val successSnackbars = remember { SnackbarHostState() }
    val constraints = LocalChatEngineConstraints.current

    val addressRegex = remember(constraints) {
        Regex("[${constraints.uidCharacterRange}]+")
    }
    
    // View model action handler
    LaunchedEffect(Unit) {
        viewModel.actionData.collect { action ->
            when (action) {
                is ExitDialog -> onDismiss()
                is ShowError -> {
                    hasAddressError = true
                    errorSnackbars.show(Res.string.add_friend_dialog_error_message)
                }
            }
        }
    }
    
    val defaultName = stringResource(Res.string.friend_request)
    SimpleDialog(
        title = Res.string.add_friend_dialog_title,
        isConfirmEnabled = addressRegex.matches(address.text) && message.text.isNotBlank(),
        onConfirm = {
            viewModel.addFriend(
                name = name.text.takeIfNotBlank() ?: defaultName,
                address = address.text,
                message = message.text,
            )
        },
        onCancel = onDismiss,
    ) {
        Column(
            verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingSmall),
        ) {
            SimpleTextField(
                value = name,
                onValueChange = { name = it.takeIfUnderLength(constraints.maxNameLength, name) },
                labelText = stringResource(Res.string.add_friend_dialog_name_label),
                keyboardOptions = KeyboardOptions(
                    autoCorrectEnabled = false,
                    imeAction = ImeAction.Next
                ),
                modifier = Modifier.fillMaxWidth(),
            )
            ContactAddressInputField(
                address = address,
                onValueChange = { address = it },
                isPublicAddress = false,
                successSnackbar = successSnackbars,
                errorSnackbar = errorSnackbars,
                isError = hasAddressError,
            )
            SimpleTextField(
                value = message,
                onValueChange = { message = it.takeIfUnderLength(constraints.maxFriendRequestLength, message) },
                labelText = stringResource(Res.string.add_friend_dialog_message_label),
                modifier = Modifier.fillMaxWidth(),
            )
        }
        SimpleSnackbarHost(
            state = errorSnackbars,
            backgroundColor = ChatTheme.colors.error,
            modifier = Modifier.align(Alignment.BottomCenter),
        )
        SimpleSnackbarHost(
            state = successSnackbars,
            backgroundColor = ChatTheme.colors.secondary,
            modifier = Modifier.align(Alignment.BottomCenter),
        )
    }
}