package nand.apps.chat.ui.groups.settings

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Lock
import androidx.compose.material.icons.outlined.LockOpen
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.KeyboardType
import nand.apps.chat.model.event.ErrorDialogEvent
import nand.apps.chat.model.group.ChatGroupData
import nand.apps.chat.model.group.ChatGroupFeature
import nand.apps.chat.model.group.ChatGroupTalkState
import nand.apps.chat.model.user.UserData
import nand.apps.chat.ui.button.IconButton
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.provider.LocalChatEngineConstraints
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.text.*
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.viewmodel.composableViewModel
import nand.apps.chat.util.*
import org.jetbrains.compose.resources.stringResource
import seers.composeapp.generated.resources.*

private const val DEFAULT_PEER_LIMIT = 100 // Matches MAX_GC_PEERS_DEFAULT
private const val MAX_PEER_LIMIT = 65536

@Composable
fun GroupSettingsDialog(
    group: ChatGroupData,
    selfPeer: UserData,
    onDismiss: () -> Unit,
    viewModel: GroupSettingsDialogViewModel = composableViewModel()
) {
    val constraints = LocalChatEngineConstraints.current
    val eventHandler = LocalEventHandler.current
    var title by rememberTextFieldValue(group.title)
    var description by rememberTextFieldValue(group.description)
    var isDescriptionLocked by rememberSaveable { mutableStateOf(group.isDescriptionLocked) }
    var password by rememberTextFieldValue(group.password.orEmpty())
    var peerLimit by rememberTextFieldValue(group.peerLimit.toString())
    var talkState by remember { mutableStateOf(group.talkState) }
    var isTalkStateDialogVisible by rememberSaveable { mutableStateOf(false) }
    var inputError by remember { mutableStateOf<GroupSettingsErrorType?>(null) }
    SimpleDialog(
        title = Res.string.group_settings,
        onConfirm = {
            inputError = when {
                group.isTitleEditable && title.text.isBlank() -> {
                    eventHandler += ErrorDialogEvent(getStringBlocking(Res.string.group_error_title_blank))
                    GroupSettingsErrorType.TITLE
                }
                ChatGroupFeature.PEER_LIMIT in group.features && selfPeer.isFounder && peerLimit.text.toIntOrZero() !in 1..MAX_PEER_LIMIT -> {
                    eventHandler += ErrorDialogEvent(getStringBlocking(Res.string.group_error_invalid_peer_limit))
                    GroupSettingsErrorType.PEER_LIMIT
                }
                else -> {
                    viewModel.update(
                        group.copy(
                            title = title.text,
                            description = description.text,
                            isDescriptionLocked = isDescriptionLocked,
                            password = password.text.takeIfNotBlank(),
                            peerLimit = peerLimit.text.toIntOrNull()?.takeIf { it > 0 } ?: DEFAULT_PEER_LIMIT,
                            talkState = talkState,
                        )
                    )
                    onDismiss()
                    null
                }
            }
        },
        onCancel = onDismiss,
    ) {
        Column(
            verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingSmall),
            modifier = Modifier
                .fillMaxWidth()
                .verticalScroll(state = rememberScrollState()),
        ) {
            SimpleTextField(
                value = title,
                onValueChange = { title = it.takeIfUnderLength(constraints.maxGroupNameLength, title) },
                labelText = stringResource(Res.string.group_title),
                isEnabled = group.isTitleEditable,
                isError = inputError == GroupSettingsErrorType.TITLE,
                modifier = Modifier.fillMaxWidth(),
            )
            if (group.isDescEditable) {
                SimpleTextField(
                    value = description,
                    labelText = stringResource(Res.string.group_description),
                    onValueChange = { description = it.takeIfUnderLength(constraints.maxGroupDescLength, description) },
                    isEnabled = selfPeer.isStaffRole || !group.isDescriptionLocked,
                    trailingIcon = {
                        IconButton(
                            icon = when {
                                isDescriptionLocked -> Icons.Outlined.Lock
                                else -> Icons.Outlined.LockOpen
                            },
                            contentDescription = stringResource(
                                when {
                                    isDescriptionLocked -> Res.string.group_lock_description
                                    else -> Res.string.group_unlock_description
                                }
                            ),
                            color = when {
                                isDescriptionLocked -> ChatTheme.colors.onSurface
                                else -> ChatTheme.colors.onBackground2
                            },
                            onClick = { isDescriptionLocked = !isDescriptionLocked },
                            isEnabled = selfPeer.isFounder,
                        )
                    },
                    modifier = Modifier.fillMaxWidth(),
                )
            }
            if (ChatGroupFeature.PASSWORD in group.features && selfPeer.isFounder) {
                SimplePasswordField(
                    password = password,
                    onValueChange = {
                        password = it.takeIfUnderLength(constraints.maxGroupPasswordLength, password)
                    },
                    label = Res.string.group_password,
                    modifier = Modifier.fillMaxWidth(),
                )
            }
            if (ChatGroupFeature.PEER_LIMIT in group.features) {
                SimpleTextField(
                    value = peerLimit,
                    onValueChange = { peerLimit = it.copy(text = it.text.replace(REGEX_NON_DIGITS, "")) },
                    labelText = stringResource(Res.string.group_peer_limit),
                    keyboardOptions = KeyboardOptions(
                        autoCorrectEnabled = false,
                        keyboardType = KeyboardType.Number,
                    ),
                    isEnabled = selfPeer.isFounder,
                    isError = inputError == GroupSettingsErrorType.PEER_LIMIT,
                    modifier = Modifier.fillMaxWidth(),
                )
            }
            if (group.hasRoles) {
                ClickableTextField(
                    text = talkState.getLabel(),
                    label = stringResource(Res.string.group_talk_state_label),
                    onClick = { isTalkStateDialogVisible = true },
                    isEnabled = selfPeer.isFounder,
                )
            }
        }
    }
    if (isTalkStateDialogVisible) {
        SimpleDialog(
            title = Res.string.group_talk_state_label,
            confirmLabel = null,
            onCancel = { isTalkStateDialogVisible = false },
        ) {
            Column {
                ChatGroupTalkState.entries.forEach { state ->
                    SimpleTextRadioButton(
                        text = state.getLabel(),
                        selected = talkState == state,
                        onClick = {
                            talkState = state
                            isTalkStateDialogVisible = false
                        },
                    )
                }
            }
        }
    }
}

private enum class GroupSettingsErrorType {
    TITLE,
    PEER_LIMIT,
}