package nand.apps.chat.ui.settings.filetransfer

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Add
import androidx.compose.material.icons.outlined.Delete
import androidx.compose.material.icons.outlined.FileOpen
import androidx.compose.material.icons.outlined.Share
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.TextFieldValue
import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import nand.apps.chat.io.ChatFileBrowser
import nand.apps.chat.io.ChatFileSystem
import nand.apps.chat.io.toSanitizedFileName
import nand.apps.chat.model.settings.filetransfer.FileTransferUploadConfig
import nand.apps.chat.model.settings.filetransfer.FileTransferUploadMethod
import nand.apps.chat.ui.button.IconButton
import nand.apps.chat.ui.button.SimpleTextButton
import nand.apps.chat.ui.file.ChatFilePickerDialog
import nand.apps.chat.ui.file.ChatFilePickerOptions
import nand.apps.chat.ui.settings.*
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.SimpleTextFieldButton
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.util.REGEX_NON_DIGITS
import nand.apps.chat.util.remove
import nand.apps.chat.util.toLongOrZero
import okio.Path.Companion.toPath
import org.jetbrains.compose.resources.stringResource
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*
import kotlin.uuid.ExperimentalUuidApi
import kotlin.uuid.Uuid

private const val MEGABYTE = 1_000_000
private const val CONFIG_EXTENSION = "json"

@Composable
fun FileTransferUploadConfigScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) {
    val state = viewModel.settings.fileSettings
    val config = state.uploadConfig
    var label by rememberTextFieldValue(config.label)
    var url by rememberTextFieldValue(config.url)
    var method by remember { mutableStateOf(config.method) }
    var maxFileSize by rememberTextFieldValue((config.maxFileSize / MEGABYTE).toString())
    var formFileNameKey by rememberTextFieldValue(config.formFileNameKey)
    val formFields = remember { config.formFields.map { FormField(it.key, it.value) }.toMutableStateList() }

    val fileSystem: ChatFileSystem by getKoin().inject()
    val fileBrowser: ChatFileBrowser by getKoin().inject()
    var filePickerOptions by remember { mutableStateOf<ChatFilePickerOptions?>(null) }
    
    val getUploadConfig = {
        FileTransferUploadConfig(
            label = label.text,
            url = url.text,
            method = method,
            maxFileSize = maxFileSize.text.toLongOrZero() * MEGABYTE,
            formFileNameKey = formFileNameKey.text,
            formFields = formFields.filter { it.key.isNotBlank() }.associate { it.key to it.value },
        )
    }

    val readConfig: (String) -> Unit = { path ->
        fileSystem.readConfigFromFile(path)?.let { config ->
            label = TextFieldValue(config.label)
            url = TextFieldValue(config.url)
            method = config.method
            maxFileSize = TextFieldValue((config.maxFileSize / MEGABYTE).toString())
            formFileNameKey = TextFieldValue(config.formFileNameKey)
            formFields.clear()
            config.formFields.forEach { (key, value) -> formFields += FormField(key, value) }
        }
    }

    // Save fields to view model on exit
    DisposableEffect(Unit) {
        onDispose {
            viewModel.update(state.copy(uploadConfig = getUploadConfig()))
        }
    }
    
    SettingsColumn(
        title = Res.string.settings_file_transfer_upload_config_title,
        modifier = modifier,
        endContent = {
            IconButton(
                icon = Icons.Outlined.FileOpen,
                contentDescription = stringResource(Res.string.settings_file_transfer_upload_config_open),
                size = ChatTheme.dimens.iconMedium,
                onClick = {
                    if (fileBrowser.isBrowserSupported()) {
                        fileBrowser.launchBrowser { path ->
                            path?.let { readConfig(it.toString()) }
                        }
                    } else {
                        filePickerOptions = ChatFilePickerOptions(
                            extensions = setOf(CONFIG_EXTENSION),
                            isOpenDialog = true,
                        )
                    }
                },
                modifier = Modifier.padding(end = ChatTheme.dimens.spacingSmall),
            )
            IconButton(
                icon = Icons.Outlined.Share,
                contentDescription = stringResource(Res.string.settings_file_transfer_upload_config_share),
                size = ChatTheme.dimens.iconMedium,
                onClick = {
                    filePickerOptions = ChatFilePickerOptions(
                        fileName = label.text.toSanitizedFileName() + "." + CONFIG_EXTENSION,
                        extensions = setOf(CONFIG_EXTENSION),
                        isOpenDialog = false,
                    )
                },
            )
        },
    ) {
        SimpleTextField(
            value = label,
            labelText = stringResource(Res.string.settings_file_transfer_upload_config_label),
            onValueChange = { label = it },
            keyboardOptions = KeyboardOptions(
                imeAction = ImeAction.Next,
            ),
        )
        SimpleTextField(
            value = url,
            labelText = stringResource(Res.string.settings_file_transfer_upload_config_url),
            onValueChange = { url = it },
            keyboardOptions = KeyboardOptions(
                autoCorrectEnabled = false,
                keyboardType = KeyboardType.Uri,
                imeAction = ImeAction.Next
            ),
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = ChatTheme.dimens.spacingSmall),
        )
        SimpleTextField(
            value = maxFileSize,
            labelText = stringResource(Res.string.settings_file_transfer_upload_config_file_size),
            onValueChange = { maxFileSize = it.remove(REGEX_NON_DIGITS) },
            keyboardOptions = KeyboardOptions(
                autoCorrectEnabled = false,
                keyboardType = KeyboardType.Number,
                imeAction = ImeAction.Next
            ),
            modifier = Modifier.padding(top = ChatTheme.dimens.spacingSmall),
        )
        Row(
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = ChatTheme.dimens.spacingMedium),
        ) {
            FileTransferUploadMethod.entries.forEach {
                SimpleTextButton(
                    text = it.name,
                    onClick = { method = it },
                    backgroundColor = if (method == it) ChatTheme.colors.secondary else ChatTheme.colors.primary,
                    modifier = Modifier.weight(1f),
                )
            }
        }

        AnimatedVisibility(
            visible = method == FileTransferUploadMethod.FORM,
            modifier = Modifier.padding(top = ChatTheme.dimens.spacingMedium)
        ) {
            Column {
                Row(
                    verticalAlignment = Alignment.CenterVertically,
                    horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
                    modifier = Modifier.fillMaxWidth(),
                ) {
                    SimpleTextField(
                        value = formFileNameKey,
                        labelText = stringResource(Res.string.settings_file_transfer_upload_config_form_file_name),
                        onValueChange = { formFileNameKey = it },
                        keyboardOptions = KeyboardOptions(autoCorrectEnabled = false),
                        modifier = Modifier.weight(1f),
                    )
                    SimpleTextFieldButton(
                        buttonIcon = Icons.Outlined.Add,
                        onClick = { formFields += FormField("" ,"") },
                        modifier = Modifier.padding(top = ChatTheme.dimens.spacingMedium)
                    )
                }
                formFields.forEachIndexed { index, field ->
                    Row(
                        verticalAlignment = Alignment.CenterVertically,
                        horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(top = ChatTheme.dimens.spacingMedium),
                    ) {
                        var keyField by rememberTextFieldValue(
                            initialValue = field.key,
                            key = "Form Key ${field.uid}",
                        )
                        var valueField by rememberTextFieldValue(
                            initialValue = field.value,
                            key = "Form Value ${field.uid}",
                        )
                        SimpleTextField(
                            value = keyField,
                            labelText = stringResource(Res.string.settings_file_transfer_upload_config_form_key),
                            onValueChange = {
                                keyField = it
                                formFields[index] = formFields[index].copy(key = it.text)
                            },
                            keyboardOptions = KeyboardOptions(
                                autoCorrectEnabled = false,
                                imeAction = ImeAction.Next
                            ),
                            modifier = Modifier.weight(1f),
                        )
                        SimpleTextField(
                            value = valueField,
                            labelText = stringResource(Res.string.settings_file_transfer_upload_config_form_value),
                            onValueChange = {
                                valueField = it
                                formFields[index] = formFields[index].copy(value = it.text)
                            },
                            keyboardOptions = KeyboardOptions(autoCorrectEnabled = false),
                            modifier = Modifier.weight(1f),
                        )
                        SimpleTextFieldButton(
                            buttonIcon = Icons.Outlined.Delete,
                            onClick = { formFields.removeAt(index) },
                            modifier = Modifier.padding(top = ChatTheme.dimens.spacingMedium)
                        )
                    }
                }
            }
        }
    }

    filePickerOptions?.let { options ->
        ChatFilePickerDialog(
            options = options,
            onConfirm = { path ->
                if (options.isOpenDialog) {
                    readConfig(path)
                } else {
                    fileSystem.writeConfigToFile(getUploadConfig(), path)
                }
                filePickerOptions = null
            },
            onCancel = { filePickerOptions = null },
        )
    }
}

@OptIn(ExperimentalUuidApi::class)
private data class FormField(
    val key: String,
    val value: String,
    val uid: String = Uuid.random().toString(),
)

private val json = Json { ignoreUnknownKeys = true }

private fun ChatFileSystem.writeConfigToFile(config: FileTransferUploadConfig, path: String) {
    val directory = getDirectoryForFile(path) ?: return
    val configJson = json.encodeToString(config)
    directory.writeToFile(path.toPath().name, configJson.encodeToByteArray())
}

private fun ChatFileSystem.readConfigFromFile(path: String): FileTransferUploadConfig? {
    val directory = getDirectoryForFile(path) ?: return null
    val bytes = directory.readFromFile(path.toPath().name) ?: return null
    return try {
        json.decodeFromString<FileTransferUploadConfig>(bytes.decodeToString())
    } catch (ignore: Exception) {
        null
    }
}