package nand.apps.chat.ui.settings.net

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.text.BasicText
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.text.selection.SelectionContainer
import androidx.compose.material.Icon
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.unit.dp
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.IO
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nand.apps.chat.engine.ChatEngine
import nand.apps.chat.engine.tox.ToxChatEngine
import nand.apps.chat.text.emoji.CHAR_TO_EMOJI_CODE
import nand.apps.chat.time.formatRelativeDate
import nand.apps.chat.ui.button.IconButton
import nand.apps.chat.ui.chat.message.StylizedTextParameters
import nand.apps.chat.ui.chat.message.toStylizedMessage
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.modifier.onClick
import nand.apps.chat.ui.scrollbar.VerticalScrollbar
import nand.apps.chat.ui.settings.SettingsColumn
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.spinner.ChasingDots
import nand.apps.chat.ui.text.SimpleTextField
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.text.rememberTextFieldValue
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.tox.net.ToxBootstrapData
import nand.apps.tox.net.ToxBootstrapNode
import nand.apps.tox.net.isValidIpAddress
import org.jetbrains.compose.resources.StringResource
import org.jetbrains.compose.resources.stringResource
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*

private val NODE_LIST_COMPARATOR = Comparator<ToxBootstrapNode> { node1, node2 ->
    val onlineComp = node2.isOnline.compareTo(node1.isOnline)
    when {
        onlineComp != 0 -> onlineComp
        else -> node2.version.compareTo(node1.version)
    }
}

@OptIn(ExperimentalFoundationApi::class)
@Composable
fun ToxNodeSettingsScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier,
) {
    val settings = viewModel.settings
    val chatEngine: ChatEngine by getKoin().inject()
    val toxEngine = chatEngine as? ToxChatEngine ?: return
    var bootstrapData by remember { mutableStateOf<ToxBootstrapData?>(null) }
    var detailsNode by remember { mutableStateOf<ToxBootstrapNode?>(null) }
    var isRefreshDialogVisible by remember { mutableStateOf(false) }
    var isLoading by remember { mutableStateOf(true) }
    val coroutineScope = rememberCoroutineScope()
    LaunchedEffect(bootstrapData) {
        if (bootstrapData == null) {
            bootstrapData = withContext(Dispatchers.IO) {
                val data = toxEngine.parseBootstrapData()
                data.copy(
                    nodes = data.nodes.sortedWith(NODE_LIST_COMPARATOR)
                )
            }
            isLoading = false
        }
    }
    SettingsColumn(
        title = Res.string.settings_network_tox_nodes_title,
        isScrollable = false,
        contentPadding = 0.dp,
        modifier = modifier,
        endContent = {
            IconButton(
                icon = Icons.Outlined.Refresh,
                contentDescription = stringResource(Res.string.settings_network_tox_nodes_refresh),
                size = ChatTheme.dimens.iconMedium,
                isEnabled = !isLoading && bootstrapData != null,
                onClick = { isRefreshDialogVisible = true },
                modifier = Modifier.padding(end = ChatTheme.dimens.spacingSmall),
            )
        },
    ) {
        Box(
            modifier = Modifier
                .fillMaxWidth()
                .weight(1f),
        ) {
            if (isLoading) {
                ChasingDots(
                    modifier = Modifier.align(Alignment.Center)
                )
                return@Box
            }
            bootstrapData?.let { data ->
                val listState = rememberLazyListState()
                LazyColumn(
                    state = listState,
                    verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
                    contentPadding = PaddingValues(ChatTheme.dimens.spacingLarge),
                    modifier = Modifier
                        .clip(ChatTheme.shapes.medium)
                        .fillMaxSize(),
                ) {
                    stickyHeader {
                        TextLine(
                            text = stringResource(Res.string.settings_network_tox_nodes_last_update, formatRelativeDate(data.lastRefreshMs)),
                            style = ChatTheme.typography.body1,
                            modifier = Modifier
                                .background(ChatTheme.colors.backgroundFloating)
                                .fillMaxWidth()
                        )
                    }
                    items(
                        items = data.nodes,
                        key = { it.publicKey },
                    ) { node ->
                        Row(
                            verticalAlignment = Alignment.CenterVertically,
                            modifier = Modifier
                                .fillMaxWidth()
                                .clip(ChatTheme.shapes.medium)
                                .background(ChatTheme.colors.surface)
                                .onClick { detailsNode = node }
                                .padding(
                                    horizontal = ChatTheme.dimens.spacingMedium,
                                    vertical = ChatTheme.dimens.spacingLarge,
                                ),
                        ) {
                            if (node.location.isNotBlank()) {
                                val emoji = remember(node.location) {
                                    node.location.mapNotNull { CHAR_TO_EMOJI_CODE[it] }.joinToString(separator = "")
                                }
                                BasicText(
                                    text = emoji,
                                    style = ChatTheme.typography.h6,
                                    modifier = Modifier.padding(end = ChatTheme.dimens.spacingMedium),
                                )
                            }
                            Column(
                                modifier = Modifier.weight(1f),
                            ) {
                                TextLine(
                                    text = node.ip,
                                    style = ChatTheme.typography.body2,
                                    color = when {
                                        node.isOnline -> ChatTheme.colors.onSurface
                                        else -> ChatTheme.colors.onBackground2
                                    },
                                )
                                if (node.isOnline) {
                                    TextLine(
                                        text = when {
                                            node.isUdpOnline -> {
                                                "${stringResource(Res.string.settings_network_tox_node_udp)}: ${node.udpPort}"
                                            }
                                            node.isTcpOnline && node.tcpPorts.isNotEmpty() -> {
                                                "${stringResource(Res.string.settings_network_tox_node_tcp)}: ${node.tcpPorts.joinToString()}"
                                            }
                                            else -> ""
                                        },
                                        style = ChatTheme.typography.caption,
                                        color = ChatTheme.colors.onBackground,
                                    )
                                }
                            }
                            TextLine(
                                text = node.formattedVersion,
                                style = ChatTheme.typography.caption,
                                color = ChatTheme.colors.onBackground2,
                                modifier = Modifier.padding(end = ChatTheme.dimens.spacingMedium),
                            )
                            Icon(
                                imageVector = when {
                                    node.isOnline -> Icons.Outlined.Wifi
                                    else -> Icons.Outlined.Block
                                },
                                contentDescription = when {
                                    node.isOnline -> stringResource(Res.string.status_online)
                                    else -> stringResource(Res.string.status_offline)
                                },
                                tint = when {
                                    node.isOnline -> ChatTheme.colors.statusOnline
                                    else -> ChatTheme.colors.statusOffline
                                },
                                modifier = Modifier.size(ChatTheme.dimens.iconMedium),
                            )
                        }
                    }
                }
                VerticalScrollbar(
                    listState = listState,
                    modifier = Modifier
                        .padding(
                            top = ChatTheme.dimens.spacingLarge,
                            bottom = ChatTheme.dimens.spacingLarge,
                            end = ChatTheme.dimens.spacingXSmall,
                        )
                        .align(Alignment.CenterEnd),
                )
            }
        }
    }
    if (isRefreshDialogVisible) {
        var urlInput by rememberTextFieldValue(settings.networkSettings.toxNodesEndpointUrl)
        val updateNodes = {
            val url = urlInput.text
            viewModel.update(settings.networkSettings.copy(toxNodesEndpointUrl = url))
            isRefreshDialogVisible = false
            isLoading = true
            coroutineScope.launch(Dispatchers.IO) {
                if (toxEngine.syncBootstrapData(url)) {
                    bootstrapData = null
                }
            }
        }
        SimpleDialog(
            title = Res.string.settings_network_tox_nodes_refresh,
            onConfirm = { updateNodes() },
            onCancel = { isRefreshDialogVisible = false },
        ) {
            SimpleTextField(
                value = urlInput,
                onValueChange = { urlInput = it },
                labelText = stringResource(Res.string.settings_network_tox_nodes_endpoint_url),
                keyboardOptions = KeyboardOptions(
                    autoCorrectEnabled = false,
                    imeAction = ImeAction.Done,
                ),
                keyboardActions = KeyboardActions(
                    onDone = { updateNodes() }
                ),
                modifier = Modifier.fillMaxWidth(),
            )
        }
    }
    detailsNode?.let { node ->
        SimpleDialog(
            title = Res.string.settings_network_tox_node_details,
            confirmLabel = null,
            cancelLabel = Res.string.close,
            onCancel = { detailsNode = null },
        ) {
            val colors = ChatTheme.colors
            val tcpPorts = remember(node.tcpPorts) {
                node.tcpPorts.joinToString()
            }
            val description = remember(node.motd, settings.chatSettings, colors) {
                node.motd.toStylizedMessage(
                    params = StylizedTextParameters(
                        settings = settings.chatSettings,
                        colors = colors,
                    ),
                    removeEffectTags = true,
                ).text
            }
            SelectionContainer {
                Column(
                    verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
                ) {
                    if (node.ipv4.isValidIpAddress()) {
                        NodeDetailRow(
                            label = Res.string.settings_network_tox_node_ipv4,
                            text = node.ipv4,
                        )
                    }
                    if (node.ipv6.isValidIpAddress()) {
                        NodeDetailRow(
                            label = Res.string.settings_network_tox_node_ipv6,
                            text = node.ipv6,
                        )
                    }
                    NodeDetailRow(
                        label = Res.string.settings_network_tox_node_udp,
                        text = node.udpPort.toString(),
                    )
                    if (node.tcpPorts.isNotEmpty()) {
                        NodeDetailRow(
                            label = Res.string.settings_network_tox_node_tcp,
                            text = tcpPorts,
                        )
                    }
                    if (node.formattedVersion.isNotBlank()) {
                        NodeDetailRow(
                            label = Res.string.settings_network_tox_node_version,
                            text = node.formattedVersion,
                        )
                    }
                    NodeDetailRow(
                        label = Res.string.settings_network_tox_node_uid,
                        text = node.publicKey,
                    )
                    NodeDetailRow(
                        label = Res.string.settings_network_tox_node_owner,
                        text = node.maintainer,
                    )
                    if (node.motd.isNotBlank()) {
                        Text(
                            text = description.text,
                            color = ChatTheme.colors.onBackground,
                            style = ChatTheme.typography.body2,
                        )
                    }
                }
            }
        }
    }
}

@Composable
private fun NodeDetailRow(
    label: StringResource,
    text: String,
) = Row(
    verticalAlignment = Alignment.CenterVertically,
    modifier = Modifier.fillMaxWidth(),
) {
    TextLine(
        text = stringResource(label),
        style = ChatTheme.typography.body2,
        color = ChatTheme.colors.onBackground2,
        modifier = Modifier.weight(0.2f)
    )
    TextLine(
        text = text,
        style = ChatTheme.typography.body2,
        modifier = Modifier.weight(0.8f),
    )
}