package nand.apps.chat.ui.settings.update

import androidx.compose.foundation.layout.size
import androidx.compose.material.CircularProgressIndicator
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.*
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Modifier
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.model.event.ErrorSnackbarEvent
import nand.apps.chat.model.event.SuccessSnackbarEvent
import nand.apps.chat.model.update.AppReleaseInfo
import nand.apps.chat.time.HOUR_MILLIS
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.settings.SettingsColumn
import nand.apps.chat.ui.settings.SettingsViewModel
import nand.apps.chat.ui.settings.row.SettingsCheckboxRow
import nand.apps.chat.ui.settings.row.SettingsInputDialogRow
import nand.apps.chat.ui.settings.row.SettingsRow
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.ui.update.AppUpdateDialog
import nand.apps.chat.update.AppUpdater
import nand.apps.chat.util.REGEX_NON_DIGITS
import nand.apps.chat.util.toIntOrZero
import org.jetbrains.compose.resources.getString
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*

@Composable
fun AppUpdateSettingsScreen(
    viewModel: SettingsViewModel,
    modifier: Modifier = Modifier,
) = SettingsColumn(
    title = Res.string.settings_network_title,
    modifier = modifier,
) {
    val state = viewModel.settings.appUpdateSettingsData
    val eventHandler = LocalEventHandler.current
    val appUpdater: AppUpdater by getKoin().inject()
    var isCheckingForUpdate by rememberSaveable { mutableStateOf(false) }
    var release by remember { mutableStateOf<AppReleaseInfo?>(null) }
    LaunchedEffect(isCheckingForUpdate) {
        if (isCheckingForUpdate) {
            val latest = appUpdater.getLatestReleaseInfo()
            release = when {
                latest == null -> {
                    eventHandler += ErrorSnackbarEvent(getString(Res.string.app_updater_check_failed))
                    null
                }
                latest.isNewerRelease -> latest
                else -> {
                    eventHandler += SuccessSnackbarEvent(getString(Res.string.app_updater_up_to_date, APP_NAME))
                    null
                }
            }
            isCheckingForUpdate = false
        }
    }
    SettingsRow(
        title = Res.string.settings_app_updates_check_title,
        description = Res.string.settings_app_updates_check_desc,
        icon = Icons.Outlined.SystemUpdateAlt,
        isEnabled = !isCheckingForUpdate,
        endContent = {
            if (isCheckingForUpdate) {
                CircularProgressIndicator(
                    modifier = Modifier.size(ChatTheme.dimens.iconSmall),
                )
            }
        },
        onClick = { isCheckingForUpdate = true },
    )
    SettingsCheckboxRow(
        title = Res.string.settings_app_updates_auto_title,
        description = Res.string.settings_app_updates_auto_desc,
        icon = Icons.Outlined.FindReplace,
        isChecked = state.isAutoUpdateEnabled,
        onCheckedChange = { checked ->
            viewModel.update(state.copy(isAutoUpdateEnabled = checked))
        }
    )
    SettingsInputDialogRow(
        title = Res.string.settings_app_updates_check_interval_title,
        description = Res.string.settings_app_updates_check_interval_desc,
        label = Res.string.settings_file_transfer_avatar_expiration_label,
        icon = Icons.Outlined.Schedule,
        value = (state.updateCheckInterval / HOUR_MILLIS).toString(),
        regex = REGEX_NON_DIGITS,
        isEnabled = state.isAutoUpdateEnabled,
        onConfirm = { hours ->
            viewModel.update(state.copy(updateCheckInterval = hours.toIntOrZero() * HOUR_MILLIS))
        }
    )
    release?.let {
        AppUpdateDialog(
            release = it,
            autoDownload = false,
            onCancel = { release = null },
        )
    }
}