package nand.apps.chat.ui.update

import androidx.compose.animation.*
import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.layout.*
import androidx.compose.material.Surface
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.audio.AudioPlayer
import nand.apps.chat.audio.queueSound
import nand.apps.chat.model.event.AppUpdateDialogEvent
import nand.apps.chat.model.event.AppUpdateReadyEvent
import nand.apps.chat.model.settings.notifications.NotificationSoundType
import nand.apps.chat.model.update.AppReleaseInfo
import nand.apps.chat.platform.AppState
import nand.apps.chat.platform.AppStateProvider
import nand.apps.chat.ui.button.SimpleTextButton
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.provider.isPortraitMode
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import org.jetbrains.compose.resources.stringResource
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*

@Composable
fun BoxScope.AppUpdateBalloon() {
    val eventHandler = LocalEventHandler.current
    val audioPlayer: AudioPlayer by getKoin().inject()
    val appStateProvider: AppStateProvider by getKoin().inject()
    var release by remember { mutableStateOf<AppReleaseInfo?>(null) }
    var releaseVersionString by remember { mutableStateOf("") }
    var isDialogVisible by remember { mutableStateOf(false) }
    val isPortrait = isPortraitMode()
    val onDismiss = {
        release = null
        isDialogVisible = false
    }
    LaunchedEffect(Unit) {
        eventHandler.onEvent.collect { event ->
            when (event) {
                is AppUpdateReadyEvent -> {
                    if (appStateProvider.currentState == AppState.FOREGROUND) {
                        release = event.release
                        releaseVersionString = event.release.version.numberString
                        audioPlayer.queueSound(NotificationSoundType.NEW_MESSAGE)
                    }
                }
                is AppUpdateDialogEvent -> {
                    release = event.release
                    isDialogVisible = true
                }
                else -> Unit
            }
        }
    }
    AnimatedVisibility(
        visible = release != null && !isDialogVisible,
        enter = fadeIn() + when {
            isPortrait -> slideInVertically()
            else -> slideInHorizontally { it / 2 }
        },
        exit = fadeOut() + when {
            isPortrait -> slideOutVertically()
            else -> slideOutHorizontally { it / 2 }
        },
        modifier = Modifier
            .run { if (isPortrait) fillMaxWidth() else this }
            .padding(ChatTheme.dimens.spacingMedium)
            .run { if (isPortrait) align(Alignment.TopCenter) else align(Alignment.BottomEnd) },
    ) {
        Surface(
            shape = ChatTheme.shapes.small,
            color = ChatTheme.colors.background,
            border = BorderStroke(
                width = 2.dp,
                color = ChatTheme.colors.onBackground2.copy(alpha = 0.5f),
            ),
        ) {
            Column(
                modifier = Modifier.padding(
                    vertical = ChatTheme.dimens.spacingMedium,
                    horizontal = ChatTheme.dimens.spacingLarge,
                )
            ) {
                TextLine(
                    text = stringResource(
                        Res.string.app_updater_new_version_available,
                        APP_NAME,
                        releaseVersionString,
                    ),
                    style = ChatTheme.typography.body2.copy(fontWeight = FontWeight.Bold),
                    modifier = Modifier.padding(vertical = ChatTheme.dimens.spacingSmall)
                )
                Row(
                    verticalAlignment = Alignment.CenterVertically,
                    horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
                    modifier = Modifier
                        .padding(top = ChatTheme.dimens.spacingSmall)
                        .align(Alignment.End),
                ) {
                    SimpleTextButton(
                        text = stringResource(Res.string.dismiss),
                        onClick = onDismiss,
                    )
                    SimpleTextButton(
                        text = stringResource(Res.string.download),
                        onClick = { isDialogVisible = true },
                    )
                }
            }
        }
    }
    if (isDialogVisible) {
        release?.let {
            AppUpdateDialog(
                release = it,
                autoDownload = true,
                onCancel = onDismiss,
            )
        }
    }
}