package nand.apps.chat.ui.update

import androidx.compose.foundation.layout.*
import androidx.compose.material.Icon
import androidx.compose.material.LinearProgressIndicator
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Error
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import kotlinx.coroutines.launch
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.model.update.AppReleaseInfo
import nand.apps.chat.ui.dialog.SimpleDialog
import nand.apps.chat.ui.text.TextLine
import nand.apps.chat.ui.theme.ChatTheme
import nand.apps.chat.update.AppInstaller
import nand.apps.chat.update.AppUpdater
import okio.Path.Companion.toPath
import org.jetbrains.compose.resources.getString
import org.jetbrains.compose.resources.stringResource
import org.koin.compose.getKoin
import seers.composeapp.generated.resources.*

@Composable
fun AppUpdateDialog(
    release: AppReleaseInfo,
    autoDownload: Boolean,
    onCancel: () -> Unit,
) {
    val appUpdater: AppUpdater by getKoin().inject()
    val appInstaller: AppInstaller by getKoin().inject()
    var isBusy by rememberSaveable { mutableStateOf(false) }
    var isDownloading by rememberSaveable { mutableStateOf(autoDownload) }
    var errorMessage by rememberSaveable { mutableStateOf<String?>(null) }
    var progress by rememberSaveable { mutableStateOf(0) }
    var packagePath by rememberSaveable { mutableStateOf<String?>(null) }
    val coroutineScope = rememberCoroutineScope()
    LaunchedEffect(isDownloading) {
        if (isDownloading) {
            isBusy = true
            packagePath = appUpdater.download(
                request = release.download,
                onProgress = { progress = it },
            )?.toString() ?: run {
                errorMessage = getString(Res.string.app_updater_download_failed)
                null
            }
            isDownloading = false
            isBusy = false
        }
    }
    SimpleDialog(
        title = Res.string.app_updater_dialog_title,
        confirmLabel = when {
            packagePath != null -> Res.string.app_updater_install
            else -> Res.string.download
        },
        isConfirmEnabled = !isBusy,
        onConfirm = {
            packagePath?.let {
                coroutineScope.launch {
                    appInstaller.install(
                        release = release,
                        packagePath = it.toPath(),
                    )
                    errorMessage = getString(Res.string.app_updater_install_failed)
                }
            } ?: run {
                isDownloading = true
            }
        },
        onCancel = onCancel,
    ) {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingMedium),
            modifier = Modifier.fillMaxWidth(),
        ) {
            when {
                isDownloading -> {
                    TextLine(
                        text = stringResource(Res.string.app_updater_downloading_package, release.download.fileName),
                        style = ChatTheme.typography.body1,
                    )
                    Row(
                        verticalAlignment = Alignment.CenterVertically,
                        horizontalArrangement = Arrangement.spacedBy(ChatTheme.dimens.spacingSmall),
                    ) {
                        LinearProgressIndicator(
                            progress = progress / 100f,
                            modifier = Modifier.fillMaxWidth(),
                        )
                        TextLine(
                            text = "$progress%",
                            style = ChatTheme.typography.body1,
                        )
                    }
                }
                packagePath != null -> TextLine(
                    text = stringResource(Res.string.app_updater_download_complete),
                    style = ChatTheme.typography.body1,
                )
                errorMessage != null -> {
                    Icon(
                        imageVector = Icons.Outlined.Error,
                        contentDescription = null,
                        modifier = Modifier.size(ChatTheme.dimens.iconMedium),
                    )
                    TextLine(
                        text = errorMessage.orEmpty(),
                        style = ChatTheme.typography.body1,
                    )
                }
                else -> TextLine(
                    text = stringResource(Res.string.app_updater_new_version_available, APP_NAME, release.version.numberString),
                    style = ChatTheme.typography.body1,
                )
            }
        }
    }
}