package nand.apps.chat.io

import androidx.compose.ui.ExperimentalComposeUiApi
import androidx.compose.ui.draganddrop.DragAndDropEvent
import androidx.compose.ui.draganddrop.DragData
import androidx.compose.ui.draganddrop.awtTransferable
import androidx.compose.ui.draganddrop.dragData
import nand.apps.chat.log.getLogger
import nand.apps.chat.platform.ChatPlatform
import nand.apps.chat.platform.getPlatform
import net.thauvin.erik.urlencoder.UrlEncoderUtil
import okio.Path
import java.awt.datatransfer.DataFlavor
import java.io.IOException
import java.nio.file.Files
import java.nio.file.Paths
import java.util.zip.ZipEntry
import java.util.zip.ZipInputStream
import java.util.zip.ZipOutputStream

private const val FILE_SCHEME = "file://"

private val logger = getLogger("ChatFileUtil")

@OptIn(ExperimentalComposeUiApi::class)
actual fun DragAndDropEvent.getFilePath(): String? {
    val filePath = try {
        when {
            awtTransferable.isDataFlavorSupported(DataFlavor.stringFlavor) -> {
                (awtTransferable.getTransferData(DataFlavor.stringFlavor) as? String)?.run {
                    if (startsWith(FILE_SCHEME)) {
                        substringAfter(FILE_SCHEME).substringBefore('\n')
                    } else {
                        null
                    }
                }
            }
            else -> (dragData() as? DragData.FilesList)?.readFiles()?.firstOrNull()?.run {
                UrlEncoderUtil.decode(substringAfter("file:"))
            }
        }
    } catch (e: Exception) {
        logger.error("Failed to parse drag-and-drop event", e)
        null
    }
    return filePath?.run {
        if (getPlatform() == ChatPlatform.WINDOWS) {
            trimStart('/')
        } else {
            this
        }
    }
}

actual fun zipDirectory(directory: Path, zipPath: Path): Boolean {
    return try {
        ZipOutputStream(zipPath.toFile().outputStream()).use { stream ->
            val sp = Paths.get(directory.toString())
            Files.walk(sp)
                .filter { path -> !Files.isDirectory(path) }
                .forEach { path ->
                    val zipEntry = ZipEntry(sp.relativize(path).toString())
                    try {
                        stream.putNextEntry(zipEntry)
                        Files.copy(path, stream)
                        stream.closeEntry()
                    } catch (e: IOException) {
                        logger.error("Failed to zip directory: $directory -> $zipPath", e)
                    }
                }
        }
        zipPath.toFile().exists()
    } catch (e: Exception) {
        false
    }
}

actual fun unzipDirectory(zipPath: Path, directory: Path): Boolean {
    return try {
        ZipInputStream(zipPath.toFile().inputStream()).use { inStream ->
            while (true) {
                val entry = inStream.nextEntry ?: break
                val path = entry.name.replace("../", "/").trimStart('/')
                val outPath = directory.resolve(path)
                val outParent = outPath.parent?.toFile() ?: return false
                if (!outParent.exists() && !outParent.mkdirs()) return false
                outPath.toFile().outputStream().use { outStream ->
                    inStream.copyTo(outStream)
                }
            }
        }
        true
    } catch (e: Exception) {
        logger.error("Failed to unzip directory: $zipPath -> $directory", e)
        false
    }
}

