package nand.apps.chat.ui

import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.awt.ComposeWindow
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.*
import nand.apps.chat.BuildConfig.APP_NAME
import nand.apps.chat.BuildConfig.APP_VERSION
import nand.apps.chat.engine.ChatEngine
import nand.apps.chat.event.ChatEventHandler
import nand.apps.chat.monitor.DesktopUserActivityMonitor
import nand.apps.chat.repo.SettingsRepo
import nand.apps.chat.ui.app.App
import nand.apps.chat.ui.dispatcher.DesktopKeyEventDispatcher
import nand.apps.chat.ui.event.DesktopEventHandler
import nand.apps.chat.ui.notification.DesktopNotificationHost
import nand.apps.chat.ui.notification.UnreadMessageTracker
import nand.apps.chat.ui.provider.LocalEventHandler
import nand.apps.chat.ui.provider.LocalLoginState
import nand.apps.chat.ui.provider.LocalSettingsProvider
import nand.apps.chat.ui.window.*
import nand.apps.chat.util.roundToNearest
import org.koin.compose.getKoin
import java.awt.GraphicsEnvironment
import java.util.prefs.Preferences

const val DEFAULT_WINDOW_TITLE = "$APP_NAME v$APP_VERSION"
private const val PREF_WINDOW_WIDTH = "windowWidth"
private const val PREF_WINDOW_HEIGHT = "windowHeight"
private const val PREF_WINDOW_MAXIMIZED = "windowMaximized"
private const val DEFAULT_WINDOW_WIDTH = 760
private const val DEFAULT_WINDOW_HEIGHT = 480
private const val WINDOW_SIZE_FACTOR = 40

// TODO - Break up this function
@Composable
fun ApplicationScope.DesktopApp() {

    // Start chat engine once login has been completed
    var isAppLoaded by rememberSaveable { mutableStateOf(false) }
    var isShuttingDown by rememberSaveable { mutableStateOf(false) }
    
    // Window size
    // TODO - Move all this into a separate window size monitoring class/Composable
    val screenDevice = GraphicsEnvironment.getLocalGraphicsEnvironment().defaultScreenDevice
    val maxWindowBounds = screenDevice.getMaxWindowBounds()
    val prefs = Preferences.userNodeForPackage(javaClass)
    val windowState = rememberWindowState(
        width = prefs.getInt(PREF_WINDOW_WIDTH, DEFAULT_WINDOW_WIDTH).coerceAtMost(maxWindowBounds.width).dp,
        height = prefs.getInt(PREF_WINDOW_HEIGHT, DEFAULT_WINDOW_HEIGHT).coerceAtMost(maxWindowBounds.height).dp,
        placement = when {
            prefs.getBoolean(PREF_WINDOW_MAXIMIZED, false) -> WindowPlacement.Maximized
            else -> WindowPlacement.Floating
        },
        position = WindowPosition(Alignment.Center),
    )
    var isWindowVisible by remember { mutableStateOf(true) }
    LaunchedEffect(windowState.size, windowState.placement) {
        prefs.putInt(PREF_WINDOW_WIDTH, windowState.size.width.value.roundToNearest(WINDOW_SIZE_FACTOR))
        prefs.putInt(PREF_WINDOW_HEIGHT, windowState.size.height.value.roundToNearest(WINDOW_SIZE_FACTOR))
        prefs.putBoolean(PREF_WINDOW_MAXIMIZED, windowState.placement == WindowPlacement.Maximized)
    }

    // Unread notifications
    var unreadMessageCount by remember { mutableStateOf(0) }
    UnreadMessageTracker(
        onUnreadMessage = { unreadMessageCount++ },
        onReadMessages = { unreadMessageCount = 0 },
    )
    
    // Auto-away detection
    DesktopUserActivityMonitor()

    val koin = getKoin()
    var windowTitle by remember { mutableStateOf(DEFAULT_WINDOW_TITLE) }
    var composeWindow by remember { mutableStateOf<ComposeWindow?>(null) }
    val loggedInState = remember { mutableStateOf(false) }
    val closeWindow = {
        if (isAppLoaded) {
            val engine: ChatEngine by koin.inject()
            engine.stop()
        }
        isShuttingDown = true
    }

    DesktopEventHandler(
        onWindowTitle = { windowTitle = it },
        onAppLoaded = { isAppLoaded = true },
        onAppFocused = { event ->
            when {
                !isWindowVisible -> isWindowVisible = true
                !event.isTrayIconEvent && windowState.isMinimized -> windowState.isMinimized = false
                !event.isTrayIconEvent && composeWindow?.isActive == false -> composeWindow?.toFront()
                isWindowVisible && event.toggle -> isWindowVisible = false
            }
        },
        loggedInState = loggedInState,
    )

    val settingsRepo: SettingsRepo by koin.inject()
    val keyEventDispatcher = remember { DesktopKeyEventDispatcher() }
    val windowIcon = windowIconPainter(unreadMessageCount > 0 && settingsRepo.settings.notificationSettings.traySettings.isFlashEnabled)
    Window(
        title = (if (unreadMessageCount > 0) "*" else "") + windowTitle,
        icon = windowIcon,
        onCloseRequest = {
            if (settingsRepo.settings.notificationSettings.traySettings.isMinimizeEnabled) {
                isWindowVisible = false
            } else {
                closeWindow()
            }
        },
        state = windowState,
        visible = isWindowVisible,
        onPreviewKeyEvent = rememberShortcutListener(
            windowState = windowState,
            keyEventDispatcher = keyEventDispatcher,
        ),
    ) {
        composeWindow = window
        if (isShuttingDown) {
            DesktopShutdownScreen()
        } else {
            keyEventDispatcher.CompositionLocalProvider {
                App()
            }
        }
    }
    
    if (isAppLoaded) {
        val eventHandler: ChatEventHandler by koin.inject()
        CompositionLocalProvider(
            LocalLoginState provides loggedInState.value,
            LocalSettingsProvider provides settingsRepo.settings,
            LocalEventHandler provides eventHandler,
        ) {
            ChatWidget()
            ChatSystemTrayIcon(
                windowIcon = windowIcon,
                onClose = closeWindow,
            )
            DesktopNotificationHost(composeWindow)
            WindowFocusEventHandler(
                window = composeWindow,
                windowState = windowState,
                isWindowVisible = isWindowVisible,
            )
        }
    }
    
    LaunchedEffect(isShuttingDown) {
        if (isShuttingDown) {
            exitApplication()
        }
    }
}
